#!/usr/bin/python

import requests
import re
import json
import time
import argparse
import sys

reflink = re.compile(r'(?:>>(?:/?([a-z]+)/)?|No.)(\d+)')
full_url = re.compile(r'(?:https?://)?(?:www\.)?dobrochan\.(?:ru|org|com)/([a-z]+)/res/(\d+)\.xhtml')

def get_post(board, tid, pid):
    resp = requests.get('http://dobrochan.ru/api/post/%s/%d/%d.json' % (board, tid, pid))
    if resp.status_code != 200:
        return None
    try:
        json_resp = json.loads(resp.text)
    except ValueError:
        return None
    try:
        if json_resp['__class__'] == 'Post':
            return json_resp
        else:
            return None
    except KeyError:
        return None

def iterate_back(board, tid, to):
    last = tid
    threads = [tid]
    while len(threads) != 0:
        last = ctid = threads.pop(0)
        json_resp = get_post(board, ctid, ctid)
        if json_resp is not None and json_resp['op']:
            yield json_resp
            text = json_resp['message']
            reflinks = reflink.findall(text) + full_url.findall(text)
            for ref_board, ref_post in reflinks:
                ref_post = int(ref_post)
                if ref_board in ('', board) and ref_post < last:
                    threads.append(ref_post)
        threads.sort()
        time.sleep(to)

def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('board', type=str)
    parser.add_argument('thread', type=int)
    parser.add_argument('-o', '--outfile', type=str)
    parser.add_argument('-t', '--timeout', type=int, default=10)
    args = parser.parse_args()
    if args.outfile:
        outfile = open(args.outfile, 'w')
    else:
        outfile = sys.stdout
    total = 0
    for json_resp in iterate_back(args.board, args.thread, args.timeout):
        total += 1
        sys.stderr.write('Retrieved: %d, total %d\n' % (json_resp['display_id'], total))
        outfile.write('%s\n' % json.dumps(json_resp))
    outfile.close()

if __name__ == '__main__':
    main()